/*----------------------------------------------------------------------------------+
|                                                                                   |
|                          Simple AD Converter                                      |
|                                                                                   |
| Name: ADConveter.c                                                                |                                                            |
| Purpose:                                                                          |
|     Using 2051 as an Analog to digital converter with 64 voltage steps            |
+----------------------------------------------------------------------------------*/

/*___________ I N C L U D E S ______________________________________________________*/

#include <reg51.h>
#include <stdio.h>
#include <ctype.h>					//use toint()
#include "rprintf/rprintf.h"

sbit ADCMP	=	P3^6;
sbit Bit5	=	P1^7;
sbit Bit4	=	P1^6;
sbit Bit3	=	P1^5;
sbit Bit2	=	P1^4;
sbit Bit1	=	P1^3;
sbit Bit0	=	P1^2;

// com port with 9600 baud with crystal 11.0592MHz.
void init_uart(void)
{
	SCON  = 0x50;		        
    TMOD |= 0x20;               
    TH1   = 253;                
    TR1   = 1;                 
    TI    = 1; 
}

/*
	convert digit to analogy voltage for 89C2051 P1.0 input
*/
void digit2BinaryInput(int digit)
{
	bit b5,b4,b3,b2,b1,b0;
	b0=(digit&0x0001)/1;	//(LSB), convert interger to binary
	b1=(digit&0x0002)/2;
	b2=(digit&0x0004)/4;
	b3=(digit&0x0008)/8;
	b4=(digit&0x0010)/16;
	b5=(digit&0x0020)/32;	//(MSB)

/*
	printStr("Binary : ");		//print the binary result
	printBit(b5);
	printBit(b4);
	printBit(b3);
	printBit(b2);
	printBit(b1);
	printBit(b0);
*/
	//(MSB)														  (LSB)
	Bit5=!b5;	Bit4=!b4;	Bit3=!b3;	Bit2=!b2;	Bit1=!b1;	Bit0=!b0;
}

/*
	Main program to implement AD-convertion.
*/
void main(void)
{
	int digit=0;
	char cmd;
	char ch;

	init_uart();	// utilize rs232 port
	printStr("-----AD-Converter-----1");
	printStr("\n d. Digit to analog voltage for P1^0");
	printStr("\n s. Increment or decrement in step of analog reference voltage");
	printStr("\n c. Compare the input & reference");
	printStr("\n g. Get the digit number using analogy comparator");
	while(1)
	{
		printStr("\n[Enter Selection ] : ");
		scanf("%c",&cmd);
		switch(cmd)
		{
			case 'c':	
						if(ADCMP==1)
							printStr("\nReference greater");
						else
							printStr("\nInput greater");
						break;
			case 'd':
						printStr("\ndigit ? : ");		//a digit to analogy reference voltage 
						scanf("%d",&digit);
						digit2BinaryInput(digit);
						break;
			case 's':
						printStr("\nPress 'u' for digit increment, 'd' for decrement and 'Esc' for exit");
						printStr("\ndigit : ");
						printInt(digit);
						ch='c';
						while(ch!=0x1B)					//0x1B=ESC
						{							
							printStr("\r");
							scanf("%c",&ch);
						
							if(ch=='u')					//check up or down
								if(digit<63)
									digit++;
								else
									digit=63;
							if(ch=='d')
								if(digit>0)
									digit--;
								else 
									digit=0;

							printStr("\rdigit : ");		//screen out
							if(digit<10)				//for 2 digit.
								printStr(" ");
							printInt(digit);
							printStr("  ");
							digit2BinaryInput(digit);
						}
						break;
			case 'g':
						digit=0;						//start at 0
						printStr("\n");
						do								//if input > reference, increase reference
						{								//voltage by step increment
							digit2BinaryInput(digit);
							digit++;
						}while(!ADCMP && digit<=63);
						digit--;
						printStr("\nThe reference digit is : ");
						printInt(digit);
						break;
		   default:
		   				printStr("\nInvalid input");
						break;
		}//endofswitch	
	}//endofwhile
}

